# py/pyext - python script objects for PD and MaxMSP
#
# Copyright (c) 2002-2007 Thomas Grill (gr@grrrr.org)
# For information on usage and redistribution, and for a DISCLAIMER OF ALL
# WARRANTIES, see the file, "license.txt," in this distribution.  
#

try:
	import pyext
except:
	print "ERROR: This script must be loaded by the PD/Max pyext external"

#################################################################

# helper function - determine whether argument is a numeric type
def isNumber(value):
	import types
	if type(value) in (types.FloatType, types.IntType, types.LongType):
		return 1
	else:
		return 0

class acc_filter(pyext._class):
	"""Filter out Wiimote accelerometer data"""

	# number of inlets and outlets
	# inlets: 1 = delta, 2 = pitch, 3 = roll, 4 = x, 5 = y, 6 = z
	# outlets: 1 = avg delta, 2 = avg pitch, 3 = avg roll,
	#		   4 = value over threshold, 5 = x delta, 6 = y delta, 7 = z delta
	#_inlets=6
	#_outlets=7

	# Constructor
	def __init__(self,*args):
		#accelerometer info
		self.reset_counter = 0
		self.reset_repeat = 10
		self.u_thresh = 0.4
		self.d_thresh = 0.05

		self.a_size = 50

		self.x_array = [0]
		self.delta_array = [0]
		self.delta_counter = 0

		self.y_array = [0]
		self.pitch_array = [0]
		self.pitch_counter = 0

		self.z_array = [0]
		self.roll_array = [0]
		self.roll_counter = 0

		for i in range (1, self.a_size):
			self.delta_array.append(0)
			self.pitch_array.append(0)
			self.roll_array.append(0)

	# methods for first inlet

	def float_1(self,f):
		self.delta_array[self.delta_counter] = f
		self.delta_counter = self.delta_counter + 1
		if (self.delta_counter > self.a_size-1):
			self.delta_counter = 0

		if (f > self.u_thresh):
			if (self.reset_counter == 0):
				self._outlet(4, f)

				delta_avg = sum(self.delta_array)/len(self.delta_array)
				pitch_avg = sum(self.pitch_array)/len(self.pitch_array)
				roll_avg = sum(self.roll_array)/len(self.roll_array)

				self._outlet(7, sum(self.z_array)/len(self.z_array))
				self._outlet(6, sum(self.y_array)/len(self.y_array))
				self._outlet(5, sum(self.x_array)/len(self.x_array))
				self._outlet(3, roll_avg) 
				self._outlet(2, pitch_avg) 
				self._outlet(1, delta_avg)

			self.reset_counter = self.reset_repeat

		if (f < self.d_thresh) and (self.reset_counter > 0):
			self.reset_counter -= 1

	def float_2(self,f):
		self.pitch_array[self.pitch_counter] = f
		self.pitch_counter = self.pitch_counter + 1
		if (self.pitch_counter > self.a_size-1):
			self.pitch_counter = 0

	def float_3(self,f):
		self.roll_array[self.roll_counter] = f
		self.roll_counter = self.roll_counter + 1
		if (self.roll_counter > self.a_size-1):
			self.roll_counter = 0

	def float_4(self,f):
		self.x_array.append(f)
		if (len(self.x_array) > 5):
			del self.x_array[0]

	def float_5(self,f):
		self.y_array.append(f)
		if (len(self.y_array) > 5):
			del self.y_array[0]

	def float_6(self,f):
		self.z_array.append(f)
		if (len(self.z_array) > 5):
			del self.z_array[0]

	# methods for control data

	def uthresh_1(self, f):
		if isNumber(f):
			self.u_thresh = f
			print "u_thresh = ", self.u_thresh
		else:
			print "acc_filter: ignoring uthresh garbage"		


	def dthresh_1(self, f):
		if isNumber(f):
			self.d_thresh = f
			print "d_thresh = ", self.d_thresh
		else:
			print "acc_filter: ignoring dthresh garbage"	

#################################################################

class joy_filter(pyext._class):
	"""Filter out Joystick analog data"""

	# x and y
	#_inlets = 2
	# 1-8 directions starting with 1 being left and going counter-clockwise
	#_outlets = 1

	def __init__(self,*args):
		self.x = 130
		self.y = 130

		self.output = 0

	def float_1(self,f):
		self.x = f

		#do calculations
		if (self.x > 125) and (self.x < 135):
			#x axis is at 0
			if (self.y > 125) and (self.y < 135):
				#no change
				if (self.output != 0):
					self.output = 0
					self._outlet(1, self.output)
			elif (self.y > 210):
				#up
				if (self.output != 7):
					self.output = 7
					self._outlet(1, self.output)
			elif (self.y < 50):
				#down
				if (self.output != 3):
					self.output = 3
					self._outlet(1, self.output)
		elif (self.y > 125) and (self.y < 135):
			#y axis is at 0
			if (self.x > 210):
				#right
				if (self.output != 5):
					self.output = 5
					self._outlet(1, self.output)
			elif (self.x < 50):
				#left
				if (self.output != 1):
					self.output = 1
					self._outlet(1, self.output)

		#otherwise we are dealing with diagonal versions
		elif (self.x > 190) and (self.y > 190):
			#up-right
			if (self.output != 6):
				self.output = 6
				self._outlet(1, self.output)				 
		elif (self.x > 190) and (self.y < 70):
			#down-right
			if (self.output != 4):
				self.output = 4
				self._outlet(1, self.output)				
		elif (self.x < 70) and (self.y > 190):
			#up-left
			if (self.output != 8):
				self.output = 8
				self._outlet(1, self.output)	
		elif (self.x < 70) and (self.y < 70):
			#down-left
			if (self.output != 2):
				self.output = 2
				self._outlet(1, self.output)

	def float_2(self,f):
		self.y = f

#################################################################
