# py/pyext - python script objects for PD and MaxMSP
#
# Copyright (c) 2002-2007 Thomas Grill (gr@grrrr.org)
# For information on usage and redistribution, and for a DISCLAIMER OF ALL
# WARRANTIES, see the file, "license.txt," in this distribution.  
#

try:
	import pyext
except:
	print "ERROR: This script must be loaded by the PD/Max pyext external"

#################################################################

# helper function - determine whether argument is a numeric type
def isNumber(value):
	import types
	if type(value) in (types.FloatType, types.IntType, types.LongType):
		return 1
	else:
		return 0

class acc_filter(pyext._class):
	"""Filter out Wiimote accelerometer data"""

	# number of inlets and outlets
	# inlets: 1 = delta, 2 = pitch, 3 = roll, 4 = x, 5 = y, 6 = z
	# outlets: 1 = value over threshold, 2 = median pitch, 3 = median roll,
	#		   4 = x delta, 5 = y delta, 6 = z delta
	#_inlets=6
	#_outlets=6

	# Constructor
	def __init__(self,*args):
		#accelerometer info
		self.reset_counter = 0
		self.reset_repeat = 10
		self.u_thresh = 0.4
		self.d_thresh = 0.05
		self.avg_thresh = 0.9

		self.delta = 0

		self.x_array = [0]
		self.y_array = [0]
		self.z_array = [0]
		self.xyz_size = 10

		self.pitch_array = [0]
		self.pitch_filter = [0]
		self.roll_array = [0]
		self.roll_filter = [0]

		self.a_size = 20

	# methods for first inlet

	def toggle_1(self, i):
		if (i == 1):
			tmp = self.pitch_array[len(self.pitch_array)-1]
			del self.pitch_array
			self.pitch_array = [tmp]

			tmp = self.roll_array[len(self.roll_array)-1]
			del self.roll_array
			self.roll_array = [tmp]

	def float_1(self,f):

		if (f > self.u_thresh):

			self.delta = f

			if (self.reset_counter == 0):

				self._outlet(6, sum(self.z_array)/len(self.z_array))
				self._outlet(5, sum(self.y_array)/len(self.y_array))
				self._outlet(4, sum(self.x_array)/len(self.x_array))

				tmp = sorted(self.roll_array)
				median = tmp[int(len(tmp)/2)]
				self._outlet(3, median)

				tmp = sorted(self.pitch_array)
				median = tmp[int(len(tmp)/2)]
				self._outlet(2, median)

				self._outlet(1, f)

			self.reset_counter = self.reset_repeat

		if (f < self.d_thresh) and (self.reset_counter > 0):
			self.reset_counter -= 1

	def float_2(self,f):
		tmp = sorted(self.pitch_filter)
		median = tmp[int(len(tmp)/2)]
		if (abs(f-median) < self.avg_thresh):
			self.pitch_array.append(round(f,0))
			self.pitch_filter.append(round(f,0))
			if (len(self.pitch_array) > self.a_size):
				del self.pitch_array[0]
			if (len(self.pitch_filter) > self.a_size):
				del self.pitch_filter[0]
		else:
			self.pitch_filter.append(round(f,0))
			if (len(self.pitch_filter) > self.a_size):
				del self.pitch_filter[0]		

	def float_3(self,f):
		tmp = sorted(self.roll_filter)
		median = tmp[int(len(tmp)/2)]
		if (abs(f-median) < self.avg_thresh):
			self.roll_array.append(round(f,0))
			self.roll_filter.append(round(f,0))
			if (len(self.roll_array) > self.a_size):
				del self.roll_array[0]
			if (len(self.roll_filter) > self.a_size):
				del self.roll_filter[0]
		else:
			self.roll_filter.append(round(f,0))
			if (len(self.roll_filter) > self.a_size):
				del self.roll_filter[0]	

	def float_4(self,f):
		self.x_array.append(f)
		if (len(self.x_array) > self.xyz_size):
			del self.x_array[0]

	def float_5(self,f):
		self.y_array.append(f)
		if (len(self.y_array) > self.xyz_size):
			del self.y_array[0]

	def float_6(self,f):
		self.z_array.append(f)
		if (len(self.z_array) > self.xyz_size):
			del self.z_array[0]

	# methods for control data

	def uthresh_1(self, f):
		if isNumber(f):
			self.u_thresh = f
			print "u_thresh = ", self.u_thresh
		else:
			print "acc_filter: ignoring uthresh garbage"		

	def dthresh_1(self, f):
		if isNumber(f):
			self.d_thresh = f
			print "d_thresh = ", self.d_thresh
		else:
			print "acc_filter: ignoring dthresh garbage"

	def avgthresh_1(self, f):
		if isNumber(f):
			self.avg_thresh = f
			print "avg_thresh = ", self.avg_thresh
		else:
			print "acc_filter: ignoring avgthresh garbage"

	def debug_1(self):
		tmp = sorted(self.x_array)
		median = tmp[int(len(tmp)/2)]
		print "median = ", median

#################################################################

class joy_filter(pyext._class):
	"""Filter out Joystick analog data"""

	# x and y
	#_inlets = 2
	# 1-8 directions starting with 1 being left and going counter-clockwise
	#_outlets = 1

	def __init__(self,*args):
		self.x = 130
		self.y = 130

		self.output = 0

	def float_1(self,f):
		self.x = f

		#do calculations
		if (self.x > 125) and (self.x < 135):
			#x axis is at 0
			if (self.y > 125) and (self.y < 135):
				#no change
				if (self.output != 0):
					self.output = 0
					self._outlet(1, self.output)
			elif (self.y > 210):
				#up
				if (self.output != 7):
					self.output = 7
					self._outlet(1, self.output)
			elif (self.y < 50):
				#down
				if (self.output != 3):
					self.output = 3
					self._outlet(1, self.output)
		elif (self.y > 125) and (self.y < 135):
			#y axis is at 0
			if (self.x > 210):
				#right
				if (self.output != 5):
					self.output = 5
					self._outlet(1, self.output)
			elif (self.x < 50):
				#left
				if (self.output != 1):
					self.output = 1
					self._outlet(1, self.output)

		#otherwise we are dealing with diagonal versions
		elif (self.x > 190) and (self.y > 190):
			#up-right
			if (self.output != 6):
				self.output = 6
				self._outlet(1, self.output)				 
		elif (self.x > 190) and (self.y < 70):
			#down-right
			if (self.output != 4):
				self.output = 4
				self._outlet(1, self.output)				
		elif (self.x < 70) and (self.y > 190):
			#up-left
			if (self.output != 8):
				self.output = 8
				self._outlet(1, self.output)	
		elif (self.x < 70) and (self.y < 70):
			#down-left
			if (self.output != 2):
				self.output = 2
				self._outlet(1, self.output)

	def float_2(self,f):
		self.y = f

#################################################################
